@@ -77,11 +77,13 @@ static int _yr_arena_make_ptr_relocatable(
     yr_arena_off_t base_offset,
     va_list offsets)
 {
-  yr_arena_off_t offset;
+  size_t offset;
 
   int result = ERROR_SUCCESS;
 
-  offset = (yr_arena_off_t) va_arg(offsets, yr_arena_off_t);
+  // The argument to va_arg is size_t because the offsets passed to this
+  // function are obtained with offsetof().
+  offset = va_arg(offsets, size_t);
 
   while (offset != EOL)
   {
@@ -91,7 +93,7 @@ static int _yr_arena_make_ptr_relocatable(
       return ERROR_INSUFFICIENT_MEMORY;
 
     reloc->buffer_id = buffer_id;
-    reloc->offset = base_offset + offset;
+    reloc->offset = base_offset + (yr_arena_off_t) offset;
     reloc->next = NULL;
 
     if (arena->reloc_list_head == NULL)
@@ -101,7 +103,7 @@ static int _yr_arena_make_ptr_relocatable(
       arena->reloc_list_tail->next = reloc;
 
     arena->reloc_list_tail = reloc;
-    offset = (yr_arena_off_t) va_arg(offsets, size_t);
+    offset = va_arg(offsets, size_t);
   }
 
   return result;
@@ -432,8 +434,8 @@ int yr_arena_ptr_to_ref(YR_ARENA* arena, const void* address, YR_ARENA_REF* ref)
         (uint8_t*) address < arena->buffers[i].data + arena->buffers[i].used)
     {
       ref->buffer_id = i;
-      ref->offset = (yr_arena_off_t)(
-          (uint8_t*) address - arena->buffers[i].data);
+      ref->offset =
+          (yr_arena_off_t) ((uint8_t*) address - arena->buffers[i].data);
 
       return 1;
     }
@@ -552,7 +554,7 @@ int yr_arena_load_stream(YR_STREAM* stream, YR_ARENA** arena)
 
   YR_ARENA* new_arena;
 
-  FAIL_ON_ERROR(yr_arena_create(hdr.num_buffers, 1048576, &new_arena))
+  FAIL_ON_ERROR(yr_arena_create(hdr.num_buffers, 10485, &new_arena))
 
   for (int i = 0; i < hdr.num_buffers; ++i)
   {
@@ -581,7 +583,7 @@ int yr_arena_load_stream(YR_STREAM* stream, YR_ARENA** arena)
     YR_ARENA_BUFFER* b = &new_arena->buffers[ref.buffer_id];
 
     if (ref.buffer_id >= new_arena->num_buffers ||
-        ref.offset > b->used - sizeof(void*))
+        ref.offset > b->used - sizeof(void*) || b->data == NULL)
     {
       yr_arena_release(new_arena);
       return ERROR_CORRUPT_FILE;
