@@ -179,79 +179,93 @@ static const uint8_t* jmp_if(int condition, const uint8_t* ip)
 
 static int iter_array_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
 {
-  YR_OBJECT* obj;
-
   // Check that there's two available slots in the stack, one for the next
   // item returned by the iterator and another one for the boolean that
   // indicates if there are more items.
   if (stack->sp + 1 >= stack->capacity)
     return ERROR_EXEC_STACK_OVERFLOW;
 
-  if (self->array_it.index < yr_object_array_length(self->array_it.array))
-  {
-    // Push the false value that indicates that the iterator is not exhausted.
-    stack->items[stack->sp++].i = 0;
+  // If the array that must be iterated is undefined stop the iteration right
+  // aways, as if the array would be empty.
+  if (IS_UNDEFINED(self->array_it.array))
+    goto _stop_iter;
 
-    obj = yr_object_array_get_item(
-        self->array_it.array, 0, self->array_it.index);
+  // If the current index is equal or larger than array's length the iterator
+  // has reached the end of the array.
+  if (self->array_it.index >= yr_object_array_length(self->array_it.array))
+    goto _stop_iter;
 
-    if (obj != NULL)
-      stack->items[stack->sp++].o = obj;
-    else
-      stack->items[stack->sp++].i = YR_UNDEFINED;
+  // Push the false value that indicates that the iterator is not exhausted.
+  stack->items[stack->sp++].i = 0;
 
-    self->array_it.index++;
-  }
+  YR_OBJECT* obj = yr_object_array_get_item(
+      self->array_it.array, 0, self->array_it.index);
+
+  if (obj != NULL)
+    stack->items[stack->sp++].o = obj;
   else
-  {
-    // Push true for indicating the iterator has been exhausted.
-    stack->items[stack->sp++].i = 1;
-    // Push YR_UNDEFINED as a placeholder for the next item.
     stack->items[stack->sp++].i = YR_UNDEFINED;
-  }
+
+  self->array_it.index++;
+
+  return ERROR_SUCCESS;
+
+_stop_iter:
+
+  // Push true for indicating the iterator has been exhausted.
+  stack->items[stack->sp++].i = 1;
+  // Push YR_UNDEFINED as a placeholder for the next item.
+  stack->items[stack->sp++].i = YR_UNDEFINED;
 
   return ERROR_SUCCESS;
 }
 
 static int iter_dict_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
 {
-  YR_DICTIONARY_ITEMS* items = object_as_dictionary(self->dict_it.dict)->items;
-
   // Check that there's three available slots in the stack, two for the next
   // item returned by the iterator and its key, and another one for the boolean
   // that indicates if there are more items.
   if (stack->sp + 2 >= stack->capacity)
     return ERROR_EXEC_STACK_OVERFLOW;
 
+  // If the dictionary that must be iterated is undefined, stop the iteration
+  // right away, as if the dictionary would be empty.
+  if (IS_UNDEFINED(self->dict_it.dict))
+    goto _stop_iter;
+
+  YR_DICTIONARY_ITEMS* items = object_as_dictionary(self->dict_it.dict)->items;
+
   // If the dictionary has no items or the iterator reached the last item, abort
   // the iteration, if not push the next key and value.
   if (items == NULL || self->dict_it.index == items->used)
+    goto _stop_iter;
+
+  // Push the false value that indicates that the iterator is not exhausted.
+  stack->items[stack->sp++].i = 0;
+
+  if (items->objects[self->dict_it.index].obj != NULL)
   {
-    // Push true for indicating the iterator has been exhausted.
-    stack->items[stack->sp++].i = 1;
-    // Push YR_UNDEFINED as a placeholder for the next key and value.
-    stack->items[stack->sp++].i = YR_UNDEFINED;
-    stack->items[stack->sp++].i = YR_UNDEFINED;
+    stack->items[stack->sp++].o = items->objects[self->dict_it.index].obj;
+    stack->items[stack->sp++].p = items->objects[self->dict_it.index].key;
   }
   else
   {
-    // Push the false value that indicates that the iterator is not exhausted.
-    stack->items[stack->sp++].i = 0;
-
-    if (items->objects[self->dict_it.index].obj != NULL)
-    {
-      stack->items[stack->sp++].o = items->objects[self->dict_it.index].obj;
-      stack->items[stack->sp++].p = items->objects[self->dict_it.index].key;
-    }
-    else
-    {
-      stack->items[stack->sp++].i = YR_UNDEFINED;
-      stack->items[stack->sp++].i = YR_UNDEFINED;
-    }
-
-    self->dict_it.index++;
+    stack->items[stack->sp++].i = YR_UNDEFINED;
+    stack->items[stack->sp++].i = YR_UNDEFINED;
   }
 
+  self->dict_it.index++;
+
+  return ERROR_SUCCESS;
+
+_stop_iter:
+
+  // Push true for indicating the iterator has been exhausted.
+  stack->items[stack->sp++].i = 1;
+  // Push YR_UNDEFINED as a placeholder for the next key and value.
+  stack->items[stack->sp++].i = YR_UNDEFINED;
+  stack->items[stack->sp++].i = YR_UNDEFINED;
+
   return ERROR_SUCCESS;
 }
 
@@ -312,6 +326,22 @@ static int iter_int_enum_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
   return ERROR_SUCCESS;
 }
 
+// Global table that contains the "next" function for different types of
+// iterators. The reason for using this table is to avoid storing pointers
+// in the YARA's VM stack. Instead of the pointers we store an index within
+// this table.
+static YR_ITERATOR_NEXT_FUNC iter_next_func_table[] = {
+    iter_array_next,
+    iter_dict_next,
+    iter_int_range_next,
+    iter_int_enum_next,
+};
+
+#define ITER_NEXT_ARRAY     0
+#define ITER_NEXT_DICT      1
+#define ITER_NEXT_INT_RANGE 2
+#define ITER_NEXT_INT_ENUM  3
+
 int yr_execute_code(YR_SCAN_CONTEXT* context)
 {
   YR_DEBUG_FPRINTF(2, stderr, "+ %s() {\n", __FUNCTION__);
@@ -355,7 +385,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
   uint8_t opcode;
 
-  yr_get_configuration(YR_CONFIG_STACK_SIZE, (void*) &stack.capacity);
+  yr_get_configuration_uint32(YR_CONFIG_STACK_SIZE, &stack.capacity);
 
   stack.sp = 0;
   stack.items = (YR_VALUE*) yr_malloc(stack.capacity * sizeof(YR_VALUE));
@@ -414,7 +444,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
         pop(r1);
         r2.it->array_it.array = r1.o;
         r2.it->array_it.index = 0;
-        r2.it->next = iter_array_next;
+        r2.it->next_func_idx = ITER_NEXT_ARRAY;
         push(r2);
       }
 
@@ -435,7 +465,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
         pop(r1);
         r2.it->dict_it.dict = r1.o;
         r2.it->dict_it.index = 0;
-        r2.it->next = iter_dict_next;
+        r2.it->next_func_idx = ITER_NEXT_DICT;
         push(r2);
       }
 
@@ -459,7 +489,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
         pop(r1);
         r3.it->int_range_it.next = r1.i;
         r3.it->int_range_it.last = r2.i;
-        r3.it->next = iter_int_range_next;
+        r3.it->next_func_idx = ITER_NEXT_INT_RANGE;
         push(r3);
       }
 
@@ -485,7 +515,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       {
         r3.it->int_enum_it.count = r1.i;
         r3.it->int_enum_it.next = 0;
-        r3.it->next = iter_int_enum_next;
+        r3.it->next_func_idx = ITER_NEXT_INT_ENUM;
 
         for (int64_t i = r1.i; i > 0; i--)
         {
@@ -505,11 +535,22 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       // Loads the iterator in r1, but leaves the iterator in the stack.
       pop(r1);
       push(r1);
-      // The iterator's next function is responsible for pushing the next
-      // item in the stack, and a boolean indicating if there are more items
-      // to retrieve. The boolean will be at the top of the stack after
-      // calling "next".
-      result = r1.it->next(r1.it, &stack);
+
+      if (r1.it->next_func_idx <
+          sizeof(iter_next_func_table) / sizeof(YR_ITERATOR_NEXT_FUNC))
+      {
+        // The iterator's next function is responsible for pushing the next
+        // item in the stack, and a boolean indicating if there are more items
+        // to retrieve. The boolean will be at the top of the stack after
+        // calling "next".
+        result = iter_next_func_table[r1.it->next_func_idx](r1.it, &stack);
+      }
+      else
+      {
+        // next_func_idx is outside the valid range, this should not happend.
+        result = ERROR_INTERNAL_FATAL_ERROR;
+      }
+
       stop = (result != ERROR_SUCCESS);
       break;
 
@@ -783,6 +824,12 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       push(r1);
       break;
 
+    case OP_DEFINED:
+      pop(r1);
+      r1.i = !is_undef(r1);
+      push(r1);
+      break;
+
     case OP_MOD:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_MOD: // %s()\n", __FUNCTION__);
       pop(r2);
@@ -882,7 +929,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       }
       else
       {
-        if yr_bitmask_is_set (context->rule_matches_flags, r1.i)
+        if (yr_bitmask_is_set(context->rule_matches_flags, r1.i))
           r2.i = 1;
         else
           r2.i = 0;
@@ -999,7 +1046,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
         break;
 
       case OBJECT_TYPE_FLOAT:
-        if (isnan(r1.o->value.d))
+        if (yr_isnan(r1.o->value.d))
           r1.i = YR_UNDEFINED;
         else
           r1.d = r1.o->value.d;
@@ -1115,7 +1162,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       function = object_as_function(r2.o);
       result = ERROR_INTERNAL_FATAL_ERROR;
 
-      for (int i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
+      for (i = 0; i < YR_MAX_OVERLOADED_FUNCTIONS; i++)
       {
         if (function->prototypes[i].arguments_fmt == NULL)
           break;
@@ -1245,6 +1292,40 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       push(r2);
       break;
 
+    case OP_COUNT_IN:
+      YR_DEBUG_FPRINTF(
+          2, stderr, "- case OP_COUNT_IN: // %s()\n", __FUNCTION__);
+      pop(r3);
+      pop(r2);
+      pop(r1);
+
+      ensure_defined(r1);
+      ensure_defined(r2);
+
+#if YR_PARANOID_EXEC
+      ensure_within_rules_arena(r1.p);
+#endif
+
+      match = context->matches[r3.s->idx].head;
+      r4.i = 0;
+
+      while (match != NULL)
+      {
+        if (match->base + match->offset >= r1.i &&
+            match->base + match->offset <= r2.i)
+        {
+          r4.i++;
+        }
+
+        if (match->base + match->offset > r2.i)
+          break;
+
+        match = match->next;
+      }
+
+      push(r4);
+      break;
+
     case OP_OFFSET:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_OFFSET: // %s()\n", __FUNCTION__);
       pop(r2);
@@ -1302,16 +1383,27 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       break;
 
     case OP_OF:
-      YR_DEBUG_FPRINTF(2, stderr, "- case OP_OF: // %s()\n", __FUNCTION__);
+    case OP_OF_PERCENT:
+      memcpy(&r2.i, ip, sizeof(uint64_t));
+      ip += sizeof(uint64_t);
+      assert(r2.i == OF_STRING_SET || r2.i == OF_RULE_SET);
       found = 0;
       count = 0;
       pop(r1);
 
       while (!is_undef(r1))
       {
-        if (context->matches[r1.s->idx].tail != NULL)
+        if (r2.i == OF_STRING_SET)
         {
-          found++;
+          if (context->matches[r1.s->idx].tail != NULL)
+          {
+            found++;
+          }
+        }
+        else
+        {
+          // r1.i is 1 if the rule has already matched and zero otherwise.
+          found += r1.i;
         }
         count++;
         pop(r1);
@@ -1319,10 +1411,120 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
       pop(r2);
 
+      if (opcode == OP_OF)
+      {
+        YR_DEBUG_FPRINTF(2, stderr, "- case OP_OF: // %s()\n", __FUNCTION__);
+
+        // Quantifier is "all"
+        if (is_undef(r2))
+        {
+          r1.i = found >= count ? 1 : 0;
+        }
+        // Quantifier is 0 or none. This is a special case in which we want
+        // exactly 0 strings matching. More information at:
+        // https://github.com/VirusTotal/yara/issues/1695
+        else if (r2.i == 0)
+        {
+          r1.i = found == 0 ? 1 : 0;
+        }
+        // In all other cases the number of strings matching should be at
+        // least the amount specified by the quantifier.
+        else
+        {
+          r1.i = found >= r2.i ? 1 : 0;
+        }
+      }
+      else  // OP_OF_PERCENT
+      {
+        YR_DEBUG_FPRINTF(
+            2, stderr, "- case OP_OF_PERCENT: // %s()\n", __FUNCTION__);
+
+        // If, by some weird reason, we manage to get an undefined string
+        // reference as the first thing on the stack then count would be zero.
+        // I don't know how this could ever happen but better to check for it.
+        if (is_undef(r2) || count == 0)
+          r1.i = YR_UNDEFINED;
+        else
+          r1.i = (((double) found / count) * 100) >= r2.i ? 1 : 0;
+      }
+
+      push(r1);
+      break;
+
+    case OP_OF_FOUND_IN:
+      YR_DEBUG_FPRINTF(
+          2, stderr, "- case OP_OF_FOUND_IN: // %s()\n", __FUNCTION__);
+
+      found = 0;
+      count = 0;
+
+      pop(r2);  // Offset range end
+      pop(r1);  // Offset range start
+      pop(r3);  // First string
+
+      // If any of the range boundaries are undefined the result is also
+      // undefined, be we need to unwind the stack first.
+      if (is_undef(r1) || is_undef(r2))
+      {
+        // Remove all the strings.
+        while (!is_undef(r3)) pop(r3);
+        // Remove the quantifier at the bottom of the stack.
+        pop(r3);
+        r1.i = YR_UNDEFINED;
+        push(r1);
+        break;
+      }
+
+      while (!is_undef(r3))
+      {
+#if YR_PARANOID_EXEC
+        ensure_within_rules_arena(r3.p);
+#endif
+        match = context->matches[r3.s->idx].head;
+
+        while (match != NULL)
+        {
+          // String match within range start and range end?
+          if (match->base + match->offset >= r1.i &&
+              match->base + match->offset <= r2.i)
+          {
+            found++;
+            break;
+          }
+
+          // If current match is past range end, we can stop as matches
+          // are sortred by offset in increasing order, so all remaining
+          // matches are part the range end too.
+          if (match->base + match->offset > r1.i)
+            break;
+
+          match = match->next;
+        }
+
+        count++;
+        pop(r3);
+      }
+
+      pop(r2);  // Quantifier X in expressions like "X of string_set in range"
+
+      // Quantifier is "all".
       if (is_undef(r2))
+      {
         r1.i = found >= count ? 1 : 0;
+      }
+      // Quantifier is 0 or none. This is a special case in which we want
+      // exactly 0 strings matching. More information at:
+      // https://github.com/VirusTotal/yara/issues/1695
+      else if (r2.i == 0)
+      {
+        r1.i = found == 0 ? 1 : 0;
+      }
+      // In all other cases the number of strings matching should be at least
+      // the amount specified by the quantifier.
       else
+      {
         r1.i = found >= r2.i ? 1 : 0;
+      }
 
       push(r1);
       break;
@@ -1790,6 +1992,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_ISTARTSWITH:
     case OP_ENDSWITH:
     case OP_IENDSWITH:
+    case OP_IEQUALS:
       pop(r2);
       pop(r1);
 
@@ -1828,6 +2031,11 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
             2, stderr, "- case OP_IENDSWITH: // %s()\n", __FUNCTION__);
         r1.i = ss_iendswith(r1.ss, r2.ss);
         break;
+      case OP_IEQUALS:
+        YR_DEBUG_FPRINTF(
+            2, stderr, "- case OP_IEQUALS: // %s()\n", __FUNCTION__);
+        r1.i = ss_icompare(r1.ss, r2.ss) == 0;
+        break;
       }
 
       push(r1);
