@@ -73,11 +73,6 @@ void* yr_realloc(void* ptr, size_t size)
   return (void*) HeapReAlloc(hHeap, HEAP_ZERO_MEMORY, ptr, size);
 }
 
-void yr_free(void* ptr)
-{
-  HeapFree(hHeap, 0, ptr);
-}
-
 char* yr_strdup(const char* str)
 {
   size_t len = strlen(str);
@@ -106,6 +101,11 @@ char* yr_strndup(const char* str, size_t n)
   return (char*) dup;
 }
 
+YR_API void yr_free(void* ptr)
+{
+  HeapFree(hHeap, 0, ptr);
+}
+
 #else
 
 #include <stdio.h>
@@ -137,11 +137,6 @@ void* yr_realloc(void* ptr, size_t size)
   return realloc(ptr, size);
 }
 
-void yr_free(void* ptr)
-{
-  free(ptr);
-}
-
 char* yr_strdup(const char* str)
 {
   return strdup(str);
@@ -152,4 +147,9 @@ char* yr_strndup(const char* str, size_t n)
   return strndup(str, n);
 }
 
+YR_API void yr_free(void* ptr)
+{
+  free(ptr);
+}
+
 #endif
