@@ -36,6 +36,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <yara/pe_utils.h>
 #include <yara/strutils.h>
 #include <yara/utils.h>
+#include <yara/dotnet.h>
 
 #if HAVE_LIBCRYPTO
 #include <openssl/asn1.h>
@@ -81,82 +82,39 @@ PIMAGE_NT_HEADERS32 pe_get_header(const uint8_t* data, size_t data_size)
   if (data_size < headers_size)
     return NULL;
 
-  if (yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_UNKNOWN &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_AM33 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_AMD64 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_ARM &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_ARMNT &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_ARM64 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_EBC &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_I386 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_IA64 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_M32R &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_MIPS16 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_MIPSFPU &&
-      yr_le16toh(pe_header->FileHeader.Machine) !=
-          IMAGE_FILE_MACHINE_MIPSFPU16 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_POWERPC &&
-      yr_le16toh(pe_header->FileHeader.Machine) !=
-          IMAGE_FILE_MACHINE_POWERPCFP &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_R4000 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_SH3 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_SH3DSP &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_SH4 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_SH5 &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_THUMB &&
-      yr_le16toh(pe_header->FileHeader.Machine) != IMAGE_FILE_MACHINE_WCEMIPSV2)
-  {
-    return NULL;
-  }
-
   return pe_header;
 }
 
 
 PIMAGE_DATA_DIRECTORY pe_get_directory_entry(PE* pe, int entry)
 {
-  PIMAGE_DATA_DIRECTORY directory_start;
-  uint8_t* optional_header_start;
-  uint16_t optional_header_size;
+  // Explanation
+  // https://github.com/VirusTotal/yara/issues/1525
 
-  // We are specifically NOT checking NumberOfRvaAndSizes here because it can
-  // lie. 7ff1bf680c80fd73c0b35084904848b3705480ddeb6d0eff62180bd14cd18570 has
-  // NumberOfRvaAndSizes set to 11 when in fact there is a valid
-  // IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR entry (which is more than 11). If we
-  // are overly strict here and only parse entries which are less than
-  // NumberOfRvaAndSizes we run the risk of missing otherwise perfectly valid
-  // files. Instead of being strict we check to make sure the entry is within
-  // the OptionalHeader, since SizeOfOptionalHeader includes the DataDirectory
-  // array.
+  // In Windows, any access to data directories is controlled by the
+  // RtlImageDirectoryEntryToData function. This is one of the first checks
+  // whether the desired directory entry is smaller than NumberOfRvaAndSizes.
+  // If it's not, it means that the requested directory entry is not there.
+  //
+  //  1. Must NOT check whether NumberOfRvaAndSizes > 0x10
+  //
+  //  2. Must check for NumberOfRvaAndSizes < DataDirectory except for 
+  //     IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR for 32-bit binary
+  //
+  //  3. The function must NOT check for SizeOfOptionalHeader
 
-  // In case someone requests an entry which is, by definition, invalid.
-  if (entry >= IMAGE_NUMBEROF_DIRECTORY_ENTRIES)
+
+  if ((entry != IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR || IS_64BITS_PE(pe)) &&
+      OptionalHeader(pe, NumberOfRvaAndSizes) < entry)
     return NULL;
 
-  if (IS_64BITS_PE(pe))
-  {
-    optional_header_start = (uint8_t*) &pe->header64->OptionalHeader;
-    optional_header_size = pe->header64->FileHeader.SizeOfOptionalHeader;
-    directory_start = pe->header64->OptionalHeader.DataDirectory;
-  }
-  else
-  {
-    optional_header_start = (uint8_t*) &pe->header->OptionalHeader;
-    optional_header_size = pe->header->FileHeader.SizeOfOptionalHeader;
-    directory_start = pe->header->OptionalHeader.DataDirectory;
-  }
+  PIMAGE_DATA_DIRECTORY result = &OptionalHeader(pe, DataDirectory)[entry];
 
-  // Make sure the entry doesn't point outside of the OptionalHeader.
-  if ((uint8_t*) (directory_start + entry) <=
-      optional_header_start + optional_header_size)
-  {
-    if (IS_64BITS_PE(pe))
-      return &pe->header64->OptionalHeader.DataDirectory[entry];
-    else
-      return &pe->header->OptionalHeader.DataDirectory[entry];
-  }
+  // Check that directory is in file
+  if (!struct_fits_in_pe(pe, result, IMAGE_DATA_DIRECTORY))
+    return NULL;
 
-  return NULL;
+  return result;
 }
 
 
@@ -188,6 +146,7 @@ int64_t pe_rva_to_offset(PE* pe, uint64_t rva)
       }
 
       if (rva >= yr_le32toh(section->VirtualAddress) &&
+          rva - yr_le32toh(section->VirtualAddress) < yr_le32toh(section->Misc.VirtualSize) &&
           section_rva <= yr_le32toh(section->VirtualAddress))
       {
         // Round section_offset
