@@ -131,6 +131,48 @@ void test_heuristic_quality()
       .bytes = {0x01, 0x02, 0x03, 0x04},
       .mask = {0xFF, 0xFF, 0x00, 0xFF}};
 
+  YR_ATOM a90909090 = {
+      .length = 4,
+      .bytes = {0x90, 0x90, 0x90, 0x90},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  YR_ATOM a20202020 = {
+      .length = 4,
+      .bytes = {0x20, 0x20, 0x20, 0x20},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  YR_ATOM aCCCCCCCC = {
+      .length = 4,
+      .bytes = {0xCC, 0xCC, 0xCC, 0xCC},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  YR_ATOM aFFFFFFFF = {
+      .length = 4,
+      .bytes = {0xFF, 0xFF, 0xFF, 0xFF},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  YR_ATOM a01010101 = {
+      .length = 4,
+      .bytes = {0x01, 0x01, 0x01, 0x01},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  YR_ATOM a01020102 = {
+      .length = 4,
+      .bytes = {0x01, 0x02, 0x01, 0x02},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  // abcd
+  YR_ATOM a61626364 = {
+      .length = 4,
+      .bytes = {0x61, 0x62, 0x63, 0x64},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
+  // abc.
+  YR_ATOM a6162632E = {
+      .length = 4,
+      .bytes = {0x61, 0x62, 0x63, 0x2E},
+      .mask = {0xFF, 0xFF, 0xFF, 0xFF}};
+
   c.get_atom_quality = yr_atoms_heuristic_quality;
 
   int q00000000 = yr_atoms_heuristic_quality(&c, &a00000000);
@@ -146,6 +188,14 @@ void test_heuristic_quality()
   int q000102 = yr_atoms_heuristic_quality(&c, &a000102);
   int q01020000 = yr_atoms_heuristic_quality(&c, &a01020000);
   int q0102XX04 = yr_atoms_heuristic_quality(&c, &a0102XX04);
+  int q01010101 = yr_atoms_heuristic_quality(&c, &a01010101);
+  int q01020102 = yr_atoms_heuristic_quality(&c, &a01020102);
+  int q20202020 = yr_atoms_heuristic_quality(&c, &a20202020);
+  int q90909090 = yr_atoms_heuristic_quality(&c, &a90909090);
+  int qCCCCCCCC = yr_atoms_heuristic_quality(&c, &aCCCCCCCC);
+  int qFFFFFFFF = yr_atoms_heuristic_quality(&c, &aFFFFFFFF);
+  int q61626364 = yr_atoms_heuristic_quality(&c, &a61626364);
+  int q6162632E = yr_atoms_heuristic_quality(&c, &a6162632E);
 
   a010203.mask[1] = 0x00;
 
@@ -184,6 +234,22 @@ void test_heuristic_quality()
   assert_true_expr(q010X0X > q01);
   assert_true_expr(q010X0X < q010203);
   assert_true_expr(q01020000 > q0102XX04);
+  assert_true_expr(q01020102 > q01010101);
+  assert_true_expr(q01020304 > q01020102);
+  assert_true_expr(q01020102 > q010203);
+  assert_true_expr(q01020304 > q61626364);
+  assert_true_expr(q010203 < q61626364);
+  assert_true_expr(q6162632E > q61626364);
+
+  // Byte sequences like 90 90 90 90 and CC CC CC CC are using as function
+  // padding by compilers (sequences of NOP and INT 3 instructions
+  // respectively) and therefore are very common. FF FF FF FF and 20 20 20
+  // 20 (spaces) are also very common sequences. All these common sequences
+  // are penalized and have lower qualities than 01 01 01 01.
+  assert_true_expr(q90909090 < q01010101);
+  assert_true_expr(q20202020 < q01010101);
+  assert_true_expr(qCCCCCCCC < q01010101);
+  assert_true_expr(qFFFFFFFF < q01010101);
 
   assert_true_expr(q01020304 == YR_MAX_ATOM_QUALITY);
 }
@@ -441,7 +507,7 @@ void test_atom_choose()
           {3, {0x61, 0x62, 0x63}},
       });
 
-  assert_hex_atoms("{61 6? 63 [1-5] 65 66 }", 16, atoms_61_6X_63);
+  assert_hex_atoms("{61 6? 63 [1-5] 65 66}", 16, atoms_61_6X_63);
 
   assert_hex_atoms(
       "{(61 62 63 | 65 66 67 68)}",
@@ -454,19 +520,50 @@ void test_atom_choose()
   assert_hex_atoms("{61 62 0? 64}", 16, atoms_61_62_0X_64);
 
   assert_hex_atoms(
-      "{11 ?? 11 ?? 22 33 44 55 66 }",
+      "{11 ?? 11 ?? 22 33 44 55 66}",
       1,
       (struct atom[]){
           {4, {0x22, 0x33, 0x44, 0x55}},
       });
 
-  // Test case for issue #1025
   assert_hex_atoms(
-      "{?? 11 22 33 ?? 55 66 }",
+      "{68 ?? ?? 00 00 58}",
+      1,
+      (struct atom[]){
+          {3, {0x00, 0x00, 0x58}},
+      });
+
+  assert_hex_atoms(
+      "{69 ?? a0 0d 00}",
+      1,
+      (struct atom[]){
+          {3, {0xa0, 0x0d, 0x00}},
+      });
+
+  // Test case for https://github.com/VirusTotal/yara/issues/1025
+  assert_hex_atoms(
+      "{?? 11 22 33 ?? 55 66}",
       1,
       (struct atom[]){
           {3, {0x11, 0x22, 0x33}},
       });
+
+  // Test case for https://github.com/VirusTotal/yara/issues/1646
+  assert_re_atoms(
+      "foobar\\.{128}",
+      1,
+      (struct atom[]){
+          {4, {0x62, 0x61, 0x72, 0x2e}},
+      });
+
+  // Test case for https://github.com/VirusTotal/yara/issues/1654
+  assert_hex_atoms(
+      "{(61 61 62 63 64 ?? | 65 ?? ?? 00 00 66)}",
+      2,
+      (struct atom[]){
+          {3, {0x00, 0x00, 0x66}},
+          {4, {0x61, 0x62, 0x63, 0x64}},
+      });
 }
 
 int main(int argc, char** argv)
